#options( show.error.messages=F, error = function () { cat( geterrmessage(), file=stderr() ); q( "no", 1, F ) } )

args <- commandArgs(trailingOnly = TRUE)

inFile = args[1]
outFile = args[2]
outDir = args[3]
clonalType = args[4]
species = args[5]
locus = args[6]
selection = args[7]

if (!("gridExtra" %in% rownames(installed.packages()))) {
	install.packages("gridExtra", repos="http://cran.xl-mirror.nl/") 
}
library(gridExtra)
if (!("ggplot2" %in% rownames(installed.packages()))) {
	install.packages("ggplot2", repos="http://cran.xl-mirror.nl/") 
}
library(ggplot2)
if (!("plyr" %in% rownames(installed.packages()))) {
	install.packages("plyr", repos="http://cran.xl-mirror.nl/") 
}			
library(plyr)

if (!("data.table" %in% rownames(installed.packages()))) {
	install.packages("data.table", repos="http://cran.xl-mirror.nl/") 
}
library(data.table)

if (!("reshape2" %in% rownames(installed.packages()))) {
	install.packages("reshape2", repos="http://cran.xl-mirror.nl/") 
}
library(reshape2)


test = read.table(inFile, sep="\t", header=TRUE, fill=T, comment.char="")

test = test[test$Sample != "",]

print("test1\n")

test$Top.V.Gene = gsub("[*]([0-9]+)", "", test$Top.V.Gene)
test$Top.D.Gene = gsub("[*]([0-9]+)", "", test$Top.D.Gene)
test$Top.J.Gene = gsub("[*]([0-9]+)", "", test$Top.J.Gene)

#test$VDJCDR3 = do.call(paste, c(test[c("Top.V.Gene", "Top.D.Gene", "Top.J.Gene","CDR3.Seq.DNA")], sep = ":"))
test$VDJCDR3 = do.call(paste, c(test[unlist(strsplit(clonalType, ","))], sep = ":"))

PROD = test[test$VDJ.Frame != "In-frame with stop codon" & test$VDJ.Frame != "Out-of-frame" & test$CDR3.Found.How != "NOT_FOUND" , ]
if("Functionality" %in% colnames(test)) {
	PROD = test[test$Functionality == "productive" | test$Functionality == "productive (see comment)", ]
}

NONPROD = test[test$VDJ.Frame == "In-frame with stop codon" | test$VDJ.Frame == "Out-of-frame" | test$CDR3.Found.How == "NOT_FOUND" , ]

#PRODF = PROD[ -1]

PRODF = PROD
print("test2\n")
#PRODF = unique(PRODF)
if(any(grepl(pattern="_", x=PRODF$ID))){ #dumb and way to simple
	PRODF$freq = gsub("^[0-9]+_", "", PRODF$ID)
	PRODF$freq = gsub("_.*", "", PRODF$freq)
	PRODF$freq = as.numeric(PRODF$freq)
	if(any(is.na(PRODF$freq))){ #fix the dumbness if it fails
		PRODF$freq = 1
		if(selection == "unique"){
			PRODF = PRODF[!duplicated(PRODF$VDJCDR3), ]
		}
	}
} else {
	PRODF$freq = 1
	if(selection == "unique"){
		PRODF = PRODF[!duplicated(PRODF$VDJCDR3), ]
	}
}

PRODFV = data.frame(data.table(PRODF)[, list(Length=sum(freq)), by=c("Sample", "Top.V.Gene")])
PRODFV$Length = as.numeric(PRODFV$Length)
Total = 0
Total = ddply(PRODFV, .(Sample), function(x) data.frame(Total = sum(x$Length)))
PRODFV = merge(PRODFV, Total, by.x='Sample', by.y='Sample', all.x=TRUE)
PRODFV = ddply(PRODFV, c("Sample", "Top.V.Gene"), summarise, relFreq= (Length*100 / Total))

PRODFD = data.frame(data.table(PRODF)[, list(Length=sum(freq)), by=c("Sample", "Top.D.Gene")])
PRODFD$Length = as.numeric(PRODFD$Length)
Total = 0
Total = ddply(PRODFD, .(Sample), function(x) data.frame(Total = sum(x$Length)))
PRODFD = merge(PRODFD, Total, by.x='Sample', by.y='Sample', all.x=TRUE)
PRODFD = ddply(PRODFD, c("Sample", "Top.D.Gene"), summarise, relFreq= (Length*100 / Total))

PRODFJ = data.frame(data.table(PRODF)[, list(Length=sum(freq)), by=c("Sample", "Top.J.Gene")])
PRODFJ$Length = as.numeric(PRODFJ$Length)
Total = 0
Total = ddply(PRODFJ, .(Sample), function(x) data.frame(Total = sum(x$Length)))
PRODFJ = merge(PRODFJ, Total, by.x='Sample', by.y='Sample', all.x=TRUE)
PRODFJ = ddply(PRODFJ, c("Sample", "Top.J.Gene"), summarise, relFreq= (Length*100 / Total))

V = c("v.name\tchr.orderV\n")
D = c("v.name\tchr.orderD\n")	
J = c("v.name\tchr.orderJ\n")

print("test3\n")

if(species == "human"){
	if(locus == "trb"){		
		V = c("v.name\tchr.orderV\nTRBV2\t1\nTRBV3-1\t2\nTRBV4-1\t3\nTRBV5-1\t4\nTRBV6-1\t5\nTRBV4-2\t6\nTRBV6-2\t7\nTRBV4-3\t8\nTRBV6-3\t9\nTRBV7-2\t10\nTRBV6-4\t11\nTRBV7-3\t12\nTRBV9\t13\nTRBV10-1\t14\nTRBV11-1\t15\nTRBV10-2\t16\nTRBV11-2\t17\nTRBV6-5\t18\nTRBV7-4\t19\nTRBV5-4\t20\nTRBV6-6\t21\nTRBV5-5\t22\nTRBV7-6\t23\nTRBV5-6\t24\nTRBV6-8\t25\nTRBV7-7\t26\nTRBV6-9\t27\nTRBV7-8\t28\nTRBV5-8\t29\nTRBV7-9\t30\nTRBV13\t31\nTRBV10-3\t32\nTRBV11-3\t33\nTRBV12-3\t34\nTRBV12-4\t35\nTRBV12-5\t36\nTRBV14\t37\nTRBV15\t38\nTRBV16\t39\nTRBV18\t40\nTRBV19\t41\nTRBV20-1\t42\nTRBV24-1\t43\nTRBV25-1\t44\nTRBV27\t45\nTRBV28\t46\nTRBV29-1\t47\nTRBV30\t48")
		D = c("v.name\tchr.orderD\nTRBD1\t1\nTRBD2\t2\n")	
		J = c("v.name\tchr.orderJ\nTRBJ1-1\t1\nTRBJ1-2\t2\nTRBJ1-3\t3\nTRBJ1-4\t4\nTRBJ1-5\t5\nTRBJ1-6\t6\nTRBJ2-1\t7\nTRBJ2-2\t8\nTRBJ2-3\t9\nTRBJ2-4\t10\nTRBJ2-5\t11\nTRBJ2-6\t12\nTRBJ2-7\t13")
	} else if (locus == "tra"){
		V = c("v.name\tchr.orderVTRAV1-1\t1\nTRAV1-2\t2\nTRAV2\t3\nTRAV3\t4\nTRAV4\t5\nTRAV5\t6\nTRAV6\t7\nTRAV7\t8\nTRAV8-1\t9\nTRAV9-1\t10\nTRAV10\t11\nTRAV12-1\t12\nTRAV8-2\t13\nTRAV8-3\t14\nTRAV13-1\t15\nTRAV12-2\t16\nTRAV8-4\t17\nTRAV13-2\t18\nTRAV14/DV4\t19\nTRAV9-2\t20\nTRAV12-3\t21\nTRAV8-6\t22\nTRAV16\t23\nTRAV17\t24\nTRAV18\t25\nTRAV19\t26\nTRAV20\t27\nTRAV21\t28\nTRAV22\t29\nTRAV23/DV6\t30\nTRAV24\t31\nTRAV25\t32\nTRAV26-1\t33\nTRAV27\t34\nTRAV29/DV5\t35\nTRAV30\t36\nTRAV26-2\t37\nTRAV34\t38\nTRAV35\t39\nTRAV36/DV7\t40\nTRAV38-1\t41\nTRAV38-2/DV8\t42\nTRAV39\t43\nTRAV40\t44\nTRAV41\t45\n")
		D = c("v.name\tchr.orderD\n")	
		J = c("v.name\tchr.orderJ\nTRAJ57\t1\nTRAJ56\t2\nTRAJ54\t3\nTRAJ53\t4\nTRAJ52\t5\nTRAJ50\t6\nTRAJ49\t7\nTRAJ48\t8\nTRAJ47\t9\nTRAJ46\t10\nTRAJ45\t11\nTRAJ44\t12\nTRAJ43\t13\nTRAJ42\t14\nTRAJ41\t15\nTRAJ40\t16\nTRAJ39\t17\nTRAJ38\t18\nTRAJ37\t19\nTRAJ36\t20\nTRAJ34\t21\nTRAJ33\t22\nTRAJ32\t23\nTRAJ31\t24\nTRAJ30\t25\nTRAJ29\t26\nTRAJ28\t27\nTRAJ27\t28\nTRAJ26\t29\nTRAJ24\t30\nTRAJ23\t31\nTRAJ22\t32\nTRAJ21\t33\nTRAJ20\t34\nTRAJ18\t35\nTRAJ17\t36\nTRAJ16\t37\nTRAJ15\t38\nTRAJ14\t39\nTRAJ13\t40\nTRAJ12\t41\nTRAJ11\t42\nTRAJ10\t43\nTRAJ9\t44\nTRAJ8\t45\nTRAJ7\t46\nTRAJ6\t47\nTRAJ5\t48\nTRAJ4\t49\nTRAJ3\t50")
	} else if (locus == "trg"){
		V = c("v.name\tchr.orderV\nTRGV9\t1\nTRGV8\t2\nTRGV5\t3\nTRGV4\t4\nTRGV3\t5\nTRGV2\t6")
		D = c("v.name\tchr.orderD\n")	
		J = c("v.name\tchr.orderJ\nTRGJ2\t1\nTRGJP2\t2\nTRGJ1\t3\nTRGJP1\t4")
	} else if (locus == "trd"){
		V = c("v.name\tchr.orderV\nTRDV1\t1\nTRDV2\t2\nTRDV3\t3")
		D = c("v.name\tchr.orderD\nTRDD1\t1\nTRDD2\t2\nTRDD3\t3")	
		J = c("v.name\tchr.orderJ\nTRDJ1\t1\nTRDJ4\t2\nTRDJ2\t3\nTRDJ3\t4")
	}
} else if (species == "mouse"){
	if(locus == "trb"){		
		cat("mouse trb not yet implemented")
	} else if (locus == "tra"){
		cat("mouse tra not yet implemented")
	} else if (locus == "trg"){
		cat("mouse trg not yet implemented")
	} else if (locus == "trd"){
		cat("mouse trd not yet implemented")
	}
}
useD = TRUE
if(species == "human" && locus == "tra"){
	useD = FALSE
	cat("No D Genes in this species/locus")
}

print("test4\n")

tcV = textConnection(V)
Vchain = read.table(tcV, sep="\t", header=TRUE)
PRODFV = merge(PRODFV, Vchain, by.x='Top.V.Gene', by.y='v.name', all.x=TRUE)
close(tcV)


tcD = textConnection(D)
Dchain = read.table(tcD, sep="\t", header=TRUE)
PRODFD = merge(PRODFD, Dchain, by.x='Top.D.Gene', by.y='v.name', all.x=TRUE)
close(tcD)



tcJ = textConnection(J)
Jchain = read.table(tcJ, sep="\t", header=TRUE)
PRODFJ = merge(PRODFJ, Jchain, by.x='Top.J.Gene', by.y='v.name', all.x=TRUE)
close(tcJ)

setwd(outDir)

write.table(PRODF, "allUnique.csv", sep=",",quote=F,row.names=F,col.names=T)

pV = ggplot(PRODFV)
pV = pV + geom_bar( aes( x=factor(reorder(Top.V.Gene, chr.orderV)), y=relFreq, fill=Sample), stat='identity', position="dodge") + theme(axis.text.x = element_text(angle = 90, hjust = 1))
pV = pV + xlab("Summary of V gene") + ylab("Frequency") + ggtitle("Relative frequency of V gene usage")
write.table(x=PRODFV, file="VFrequency.csv", sep=",",quote=F,row.names=F,col.names=T)

png("VPlot.png",width = 1280, height = 720)
pV
dev.off();

pD = ggplot(PRODFD)
pD = pD + geom_bar( aes( x=factor(reorder(Top.D.Gene, chr.orderD)), y=relFreq, fill=Sample), stat='identity', position="dodge") + theme(axis.text.x = element_text(angle = 90, hjust = 1))
pD = pD + xlab("Summary of D gene") + ylab("Frequency") + ggtitle("Relative frequency of D gene usage")
write.table(x=PRODFD, file="DFrequency.csv", sep=",",quote=F,row.names=F,col.names=T)

png("DPlot.png",width = 800, height = 600)
pD
dev.off();

pJ = ggplot(PRODFJ)
pJ = pJ + geom_bar( aes( x=factor(reorder(Top.J.Gene, chr.orderJ)), y=relFreq, fill=Sample), stat='identity', position="dodge") + theme(axis.text.x = element_text(angle = 90, hjust = 1))
pJ = pJ + xlab("Summary of J gene") + ylab("Frequency") + ggtitle("Relative frequency of J gene usage")
write.table(x=PRODFJ, file="JFrequency.csv", sep=",",quote=F,row.names=F,col.names=T)

png("JPlot.png",width = 800, height = 600)
pJ
dev.off();

print("test5\n")

VGenes = PRODF[,c("Sample", "Top.V.Gene", "freq")]
VGenes$Top.V.Gene = gsub("-.*", "", VGenes$Top.V.Gene)
VGenes = data.frame(data.table(VGenes)[, list(Count=sum(freq)), by=c("Sample", "Top.V.Gene")])
TotalPerSample = data.frame(data.table(VGenes)[, list(total=sum(.SD$Count)), by=Sample])
VGenes = merge(VGenes, TotalPerSample, by="Sample")
VGenes$Frequency = VGenes$Count * 100 / VGenes$total
VPlot = ggplot(VGenes)
VPlot = VPlot + geom_bar(aes( x = Top.V.Gene, y = Frequency, fill = Sample), stat='identity', position='dodge' ) + theme(axis.text.x = element_text(angle = 90, hjust = 1)) + 
				ggtitle("Distribution of V gene families") + 
				ylab("Percentage of sequences")
png("VFPlot.png")
VPlot
dev.off();
write.table(x=VGenes, file="VFFrequency.csv", sep=",",quote=F,row.names=F,col.names=T)

DGenes = PRODF[,c("Sample", "Top.D.Gene", "freq")]
DGenes$Top.D.Gene = gsub("-.*", "", DGenes$Top.D.Gene)
DGenes = data.frame(data.table(DGenes)[, list(Count=sum(freq)), by=c("Sample", "Top.D.Gene")])
TotalPerSample = data.frame(data.table(DGenes)[, list(total=sum(.SD$Count)), by=Sample])
DGenes = merge(DGenes, TotalPerSample, by="Sample")
DGenes$Frequency = DGenes$Count * 100 / DGenes$total
DPlot = ggplot(DGenes)
DPlot = DPlot + geom_bar(aes( x = Top.D.Gene, y = Frequency, fill = Sample), stat='identity', position='dodge' ) + theme(axis.text.x = element_text(angle = 90, hjust = 1)) + 
				ggtitle("Distribution of D gene families") + 
				ylab("Percentage of sequences")
png("DFPlot.png")
DPlot
dev.off();
write.table(x=DGenes, file="DFFrequency.csv", sep=",",quote=F,row.names=F,col.names=T)

JGenes = PRODF[,c("Sample", "Top.J.Gene", "freq")]
JGenes$Top.J.Gene = gsub("-.*", "", JGenes$Top.J.Gene)
JGenes = data.frame(data.table(JGenes)[, list(Count=sum(freq)), by=c("Sample", "Top.J.Gene")])
TotalPerSample = data.frame(data.table(JGenes)[, list(total=sum(.SD$Count)), by=Sample])
JGenes = merge(JGenes, TotalPerSample, by="Sample")
JGenes$Frequency = JGenes$Count * 100 / JGenes$total
JPlot = ggplot(JGenes)
JPlot = JPlot + geom_bar(aes( x = Top.J.Gene, y = Frequency, fill = Sample), stat='identity', position='dodge' ) + theme(axis.text.x = element_text(angle = 90, hjust = 1)) + 
				ggtitle("Distribution of J gene families") + 
				ylab("Percentage of sequences")
png("JFPlot.png")
JPlot
dev.off();
write.table(x=JGenes, file="JFFrequency.csv", sep=",",quote=F,row.names=F,col.names=T)

CDR3Length = data.frame(data.table(PRODF)[, list(Count=sum(freq)), by=c("Sample", "CDR3.Length.DNA")])
TotalPerSample = data.frame(data.table(CDR3Length)[, list(total=sum(.SD$Count)), by=Sample])
CDR3Length = merge(CDR3Length, TotalPerSample, by="Sample")
CDR3Length$Frequency = CDR3Length$Count * 100 / CDR3Length$total
CDR3LengthPlot = ggplot(CDR3Length)
CDR3LengthPlot = CDR3LengthPlot + geom_bar(aes( x = CDR3.Length.DNA, y = Frequency, fill = Sample), stat='identity', position='dodge' ) + theme(axis.text.x = element_text(angle = 90, hjust = 1)) + 
				ggtitle("Length distribution of CDR3") + 
				xlab("CDR3 Length") + 
				ylab("Percentage of sequences")
png("CDR3LengthPlot.png",width = 1280, height = 720)
CDR3LengthPlot
dev.off()
write.table(x=CDR3Length, file="CDR3LengthPlot.csv", sep=",",quote=F,row.names=F,col.names=T)

revVchain = Vchain
revDchain = Dchain
revVchain$chr.orderV = rev(revVchain$chr.orderV)
revDchain$chr.orderD = rev(revDchain$chr.orderD)

print("test6\n")

plotVD <- function(dat){
	if(length(dat[,1]) == 0){
		return()
	}
	img = ggplot() + 
	geom_tile(data=dat, aes(x=factor(reorder(Top.D.Gene, chr.orderD)), y=factor(reorder(Top.V.Gene, chr.orderV)), fill=relLength)) + 
	theme(axis.text.x = element_text(angle = 90, hjust = 1)) + 
	scale_fill_gradient(low="gold", high="blue", na.value="white") + 
	ggtitle(paste(unique(dat$Sample), " (N=" , sum(dat$Length, na.rm=T) ,")", sep="")) + 
	xlab("D genes") + 
	ylab("V Genes")
	
	png(paste("HeatmapVD_", unique(dat[3])[1,1] , ".png", sep=""), width=150+(15*length(Dchain$v.name)), height=100+(15*length(Vchain$v.name)))
	print(img)
	
	dev.off()
	write.table(x=acast(dat, Top.V.Gene~Top.D.Gene, value.var="Length"), file=paste("HeatmapVD_", unique(dat[3])[1,1], ".csv", sep=""), sep=",",quote=F,row.names=T,col.names=NA)
}

VandDCount = data.frame(data.table(PRODF)[, list(Length=sum(freq)), by=c("Top.V.Gene", "Top.D.Gene", "Sample")])

VandDCount$l = log(VandDCount$Length)
maxVD = data.frame(data.table(VandDCount)[, list(max=max(l)), by=c("Sample")])
VandDCount = merge(VandDCount, maxVD, by.x="Sample", by.y="Sample", all.x=T)
VandDCount$relLength = VandDCount$l / VandDCount$max

cartegianProductVD = expand.grid(Top.V.Gene = Vchain$v.name, Top.D.Gene = Dchain$v.name, Sample = unique(test$Sample))

completeVD = merge(VandDCount, cartegianProductVD, all.y=TRUE)
completeVD = merge(completeVD, revVchain, by.x="Top.V.Gene", by.y="v.name", all.x=TRUE)
completeVD = merge(completeVD, Dchain, by.x="Top.D.Gene", by.y="v.name", all.x=TRUE)
VDList = split(completeVD, f=completeVD[,"Sample"])

lapply(VDList, FUN=plotVD)

plotVJ <- function(dat){
	if(length(dat[,1]) == 0){
		return()
	}
	img = ggplot() + 
	geom_tile(data=dat, aes(x=factor(reorder(Top.J.Gene, chr.orderJ)), y=factor(reorder(Top.V.Gene, chr.orderV)), fill=relLength)) + 
	theme(axis.text.x = element_text(angle = 90, hjust = 1)) + 
	scale_fill_gradient(low="gold", high="blue", na.value="white") + 
	ggtitle(paste(unique(dat$Sample), " (N=" , sum(dat$Length, na.rm=T) ,")", sep="")) + 
	xlab("J genes") + 
	ylab("V Genes")
	
	png(paste("HeatmapVJ_", unique(dat[3])[1,1] , ".png", sep=""), width=150+(15*length(Jchain$v.name)), height=100+(15*length(Vchain$v.name)))
	print(img)
	dev.off()
	write.table(x=acast(dat, Top.V.Gene~Top.J.Gene, value.var="Length"), file=paste("HeatmapVJ_", unique(dat[3])[1,1], ".csv", sep=""), sep=",",quote=F,row.names=T,col.names=NA)
}

VandJCount = data.frame(data.table(PRODF)[, list(Length=sum(freq)), by=c("Top.V.Gene", "Top.J.Gene", "Sample")])

VandJCount$l = log(VandJCount$Length)
maxVJ = data.frame(data.table(VandJCount)[, list(max=max(l)), by=c("Sample")])
VandJCount = merge(VandJCount, maxVJ, by.x="Sample", by.y="Sample", all.x=T)
VandJCount$relLength = VandJCount$l / VandJCount$max

cartegianProductVJ = expand.grid(Top.V.Gene = Vchain$v.name, Top.J.Gene = Jchain$v.name, Sample = unique(test$Sample))

completeVJ = merge(VandJCount, cartegianProductVJ, all.y=TRUE)
completeVJ = merge(completeVJ, revVchain, by.x="Top.V.Gene", by.y="v.name", all.x=TRUE)
completeVJ = merge(completeVJ, Jchain, by.x="Top.J.Gene", by.y="v.name", all.x=TRUE)
VJList = split(completeVJ, f=completeVJ[,"Sample"])
lapply(VJList, FUN=plotVJ)

plotDJ <- function(dat){
	if(length(dat[,1]) == 0){
		return()
	}
	img = ggplot() + 
	geom_tile(data=dat, aes(x=factor(reorder(Top.J.Gene, chr.orderJ)), y=factor(reorder(Top.D.Gene, chr.orderD)), fill=relLength)) + 
	theme(axis.text.x = element_text(angle = 90, hjust = 1)) + 
	scale_fill_gradient(low="gold", high="blue", na.value="white") + 
	ggtitle(paste(unique(dat$Sample), " (N=" , sum(dat$Length, na.rm=T) ,")", sep="")) + 
	xlab("J genes") + 
	ylab("D Genes")
	
	png(paste("HeatmapDJ_", unique(dat[3])[1,1] , ".png", sep=""), width=150+(15*length(Jchain$v.name)), height=100+(15*length(Dchain$v.name)))
	print(img)
	dev.off()
	write.table(x=acast(dat, Top.D.Gene~Top.J.Gene, value.var="Length"), file=paste("HeatmapDJ_", unique(dat[3])[1,1], ".csv", sep=""), sep=",",quote=F,row.names=T,col.names=NA)
}

DandJCount = data.frame(data.table(PRODF)[, list(Length=.N), by=c("Top.D.Gene", "Top.J.Gene", "Sample")])

DandJCount$l = log(DandJCount$Length)
maxDJ = data.frame(data.table(DandJCount)[, list(max=max(l)), by=c("Sample")])
DandJCount = merge(DandJCount, maxDJ, by.x="Sample", by.y="Sample", all.x=T)
DandJCount$relLength = DandJCount$l / DandJCount$max

cartegianProductDJ = expand.grid(Top.D.Gene = Dchain$v.name, Top.J.Gene = Jchain$v.name, Sample = unique(test$Sample))

completeDJ = merge(DandJCount, cartegianProductDJ, all.y=TRUE)
completeDJ = merge(completeDJ, revDchain, by.x="Top.D.Gene", by.y="v.name", all.x=TRUE)
completeDJ = merge(completeDJ, Jchain, by.x="Top.J.Gene", by.y="v.name", all.x=TRUE)
DJList = split(completeDJ, f=completeDJ[,"Sample"])
lapply(DJList, FUN=plotDJ)

sampleFile <- file("samples.txt")
un = unique(test$Sample)
un = paste(un, sep="\n")
writeLines(un, sampleFile)
close(sampleFile)

print("test7\n")

if("Replicate" %in% colnames(test))
{
	clonalityFrame = PROD
	clonalityFrame$ReplicateConcat = do.call(paste, c(clonalityFrame[c("VDJCDR3", "Sample", "Replicate")], sep = ":"))
	clonalityFrame = clonalityFrame[!duplicated(clonalityFrame$ReplicateConcat), ]
	write.table(clonalityFrame, "clonalityComplete.csv", sep=",",quote=F,row.names=F,col.names=T)

	ClonalitySampleReplicatePrint <- function(dat){
	    write.table(dat, paste("clonality_", unique(dat$Sample) , "_", unique(dat$Replicate), ".csv", sep=""), sep=",",quote=F,row.names=F,col.names=T)
	}

	clonalityFrameSplit = split(clonalityFrame, f=clonalityFrame[,c("Sample", "Replicate")])
	#lapply(clonalityFrameSplit, FUN=ClonalitySampleReplicatePrint)

	ClonalitySamplePrint <- function(dat){
	    write.table(dat, paste("clonality_", unique(dat$Sample) , ".csv", sep=""), sep=",",quote=F,row.names=F,col.names=T)
	}

	clonalityFrameSplit = split(clonalityFrame, f=clonalityFrame[,"Sample"])
	#lapply(clonalityFrameSplit, FUN=ClonalitySamplePrint)

	clonalFreq = data.frame(data.table(clonalityFrame)[, list(Type=.N), by=c("Sample", "VDJCDR3")])
	clonalFreqCount = data.frame(data.table(clonalFreq)[, list(Count=.N), by=c("Sample", "Type")])
	clonalFreqCount$realCount = clonalFreqCount$Type * clonalFreqCount$Count
	clonalSum = data.frame(data.table(clonalFreqCount)[, list(Reads=sum(realCount)), by=c("Sample")])
	clonalFreqCount = merge(clonalFreqCount, clonalSum, by.x="Sample", by.y="Sample")

	ct = c('Type\tWeight\n2\t1\n3\t3\n4\t6\n5\t10\n6\t15')
	tcct = textConnection(ct)
	CT  = read.table(tcct, sep="\t", header=TRUE)
	close(tcct)
	clonalFreqCount = merge(clonalFreqCount, CT, by.x="Type", by.y="Type", all.x=T)
	clonalFreqCount$WeightedCount = clonalFreqCount$Count * clonalFreqCount$Weight

	ReplicateReads = data.frame(data.table(clonalityFrame)[, list(Type=.N), by=c("Sample", "Replicate", "VDJCDR3")])
	ReplicateReads = data.frame(data.table(ReplicateReads)[, list(Reads=.N), by=c("Sample", "Replicate")])
	clonalFreqCount$Reads = as.numeric(clonalFreqCount$Reads)
	ReplicateReads$squared = ReplicateReads$Reads * ReplicateReads$Reads

	ReplicatePrint <- function(dat){
		write.table(dat[-1], paste("ReplicateReads_", unique(dat[1])[1,1] , ".csv", sep=""), sep=",",quote=F,na="-",row.names=F,col.names=F)
	}

	ReplicateSplit = split(ReplicateReads, f=ReplicateReads[,"Sample"])
	lapply(ReplicateSplit, FUN=ReplicatePrint)

	ReplicateReads = data.frame(data.table(ReplicateReads)[, list(ReadsSum=sum(Reads), ReadsSquaredSum=sum(squared)), by=c("Sample")])
	clonalFreqCount = merge(clonalFreqCount, ReplicateReads, by.x="Sample", by.y="Sample", all.x=T)


	ReplicateSumPrint <- function(dat){
		write.table(dat[-1], paste("ReplicateSumReads_", unique(dat[1])[1,1] , ".csv", sep=""), sep=",",quote=F,na="-",row.names=F,col.names=F)
	}

	ReplicateSumSplit = split(ReplicateReads, f=ReplicateReads[,"Sample"])
	lapply(ReplicateSumSplit, FUN=ReplicateSumPrint)
	
	writeClonalitySequences <- function(dat){
		for(i in c(2,3,4,5,6)){
			fltr = dat[dat$Type == i,]
			if(length(fltr[,1]) == 0){
				next
			}
			tmp = clonalityFrame[clonalityFrame$Sample == fltr$Sample[1] & clonalityFrame$VDJCDR3 %in% fltr$VDJCDR3,]
			tmp = tmp[order(tmp$VDJCDR3),]
			write.table(tmp, paste("ClonalitySequences_", unique(dat[1])[1,1] , "_", i, ".csv", sep=""), sep=",",quote=F,na="-",row.names=F,col.names=T)
		}
	}
	freqsplt = split(clonalFreq[clonalFreq$Type > 1,], clonalFreq[clonalFreq$Type > 1,]$Sample)
	lapply(freqsplt, FUN=writeClonalitySequences)

	clonalFreqCountSum = data.frame(data.table(clonalFreqCount)[, list(Numerator=sum(WeightedCount, na.rm=T)), by=c("Sample")])
	clonalFreqCount = merge(clonalFreqCount, clonalFreqCountSum, by.x="Sample", by.y="Sample", all.x=T)
	clonalFreqCount$ReadsSum = as.numeric(clonalFreqCount$ReadsSum) #prevent integer overflow
	clonalFreqCount$Denominator = (((clonalFreqCount$ReadsSum * clonalFreqCount$ReadsSum) - clonalFreqCount$ReadsSquaredSum) / 2)
	clonalFreqCount$Result = (clonalFreqCount$Numerator + 1) / (clonalFreqCount$Denominator + 1)

	ClonalityScorePrint <- function(dat){
		write.table(dat$Result, paste("ClonalityScore_", unique(dat[1])[1,1] , ".csv", sep=""), sep=",",quote=F,na="-",row.names=F,col.names=F)
	}

	clonalityScore = clonalFreqCount[c("Sample", "Result")]
	clonalityScore = unique(clonalityScore)

	clonalityScoreSplit = split(clonalityScore, f=clonalityScore[,"Sample"])
	lapply(clonalityScoreSplit, FUN=ClonalityScorePrint)

	clonalityOverview = clonalFreqCount[c("Sample", "Type", "Count", "Weight", "WeightedCount")]



	ClonalityOverviewPrint <- function(dat){
		write.table(dat[-1], paste("ClonalityOverView_", unique(dat[1])[1,1] , ".csv", sep=""), sep=",",quote=F,na="-",row.names=F,col.names=F)
	}

	clonalityOverviewSplit = split(clonalityOverview, f=clonalityOverview$Sample)
	lapply(clonalityOverviewSplit, FUN=ClonalityOverviewPrint)
}

print("test8\n")

if("Functionality" %in% colnames(test))
{
	newData = data.frame(data.table(PROD)[,list(unique=.N, 
				VH.DEL=mean(X3V.REGION.trimmed.nt.nb),
				P1=mean(P3V.nt.nb),
				N1=mean(N1.REGION.nt.nb),
				P2=mean(P5D.nt.nb),
				DEL.DH=mean(X5D.REGION.trimmed.nt.nb),
				DH.DEL=mean(X3D.REGION.trimmed.nt.nb),
				P3=mean(P3D.nt.nb),
				N2=mean(N2.REGION.nt.nb),
				P4=mean(P5J.nt.nb),
				DEL.JH=mean(X5J.REGION.trimmed.nt.nb),
				Total.Del=(	mean(X3V.REGION.trimmed.nt.nb) + 
							mean(X5D.REGION.trimmed.nt.nb) + 
							mean(X3D.REGION.trimmed.nt.nb) +
							mean(X5J.REGION.trimmed.nt.nb)),
							
				Total.N=(	mean(N1.REGION.nt.nb) +
							mean(N2.REGION.nt.nb)),
							
				Total.P=(	mean(P3V.nt.nb) +
							mean(P5D.nt.nb) +
							mean(P3D.nt.nb) +
							mean(P5J.nt.nb))),
				by=c("Sample")])
	write.table(newData, "junctionAnalysis.csv" , sep=",",quote=F,na="-",row.names=F,col.names=F)
}

print("test9\n")
